from dbks.client import Client


class WorkspaceAPI:
    def __init__(self, client=Client()):
        self.client = client

    def delete(self, params=None, json=None):
        return self.client.call("POST", "/workspace/delete", json=json)

    def export(self, params=None, json=None):
        return self.client.call("GET", "/workspace/export", json=json)

    def get_status(self, params=None, json=None):
        return self.client.call("GET", "/workspace/get-status", json=json)

    def _import(self, params=None, json=None):
        return self.client.call("POST", "/workspace/import", json=json)

    def list(self, params=None, json=None):
        return self.client.call("GET", "/workspace/list", json=json)

    def mkdirs(self, params=None, json=None):
        return self.client.call("POST", "/workspace/mkdirs", json=json)



from dbks.client import Client


class ClusterAPI:
    def __init__(self, client=Client()):
        self.client = client

    def create(self, params=None, json=None):
        return self.client.call("POST", "/clusters/create", json=json)

    def edit(self, params=None, json=None):
        return self.client.call("POST", "/clusters/edit", json=json)

    def start(self, params=None, json=None):
        return self.client.call("POST", "/clusters/start", json=json)

    def restart(self, params=None, json=None):
        return self.client.call("POST", "/clusters/restart", json=json)

    def resize(self, params=None, json=None):
        return self.client.call("POST", "/clusters/resize", json=json)

    def terminate(self, params=None, json=None):
        return self.client.call("POST", "/clusters/delete", json=json)

    def delete(self, params=None, json=None):
        return self.client.call("POST", "/clusters/permanent-delete", json=json)

    def get(self, params=None, json=None):
        return self.client.call("GET", "/clusters/get", params=params)

    def pin(self, params=None, json=None):
        return self.client.call("POST", "/clusters/pin", json=json)

    def unpin(self, params=None, json=None):
        return self.client.call("POST", "/clusters/unpin", json=json)

    def list(self, params=None, json=None):
        return self.client.call("GET", "/clusters/list")

    def list_node_types(self, params=None, json=None):
        return self.client.call("GET", "/clusters/list-node-types")

    def runtime_versions(self, params=None, json=None):
        return self.client.call("GET", "/clusters/spark-versions")

    def list_zones(self, params=None, json=None):
        return self.client.call("GET", "/clusters/list-zones")

    def events(self, params=None, json=None):
        return self.client.call("POST", "/clusters/events", json=json)



from functools import partial
from requests import Response


class ResponseHandler(object):
    def __init__(self, func):
        self.func = func

    def __call__(self, instance, *args, **kwargs):
        print(f'"{instance.__class__.__name__}.{self.func.__name__}" triggered with')
        if args:
            print("args:", args)
        if kwargs:
            print("kwargs:", kwargs)
        if not args and not kwargs:
            print("no args or kwargs")
        res = self.func(instance, *args, **kwargs)
        if isinstance(res, Response):
            print("Response:")
            print("status_code:", str(res.status_code))
            # print('text:', res.text)
            print("json:", res.json())
            if str(res.status_code)[0] != "2":
                raise SystemError("Non 2XX status code returned!")
        return res

    def __get__(self, instance, owner):
        return partial(self, instance)



import os
import click


class ComplexCLI(click.MultiCommand):
    def list_commands(self, ctx):
        files = []
        for file_name in os.listdir(
            os.path.join(os.path.dirname(__file__), "cli_command")
        ):
            if file_name.endswith(".py") and not file_name.startswith("__"):
                files.append(file_name[:-3])
        files.sort()
        return files

    def get_command(self, ctx, name):
        try:
            mod = __import__(f"dbks.cli_command.{name}", None, None, ["cli"])
        except ImportError:
            return
        return mod.cli


@click.command(cls=ComplexCLI)
def cli():
    """\b
    ===============================
    ==  dbks - a Databricks CLI  ==
    ==============================="""
    pass



class Runtime:
    @staticmethod
    def v3(major: str, feature: str, ml_type: str = None, scala_version: str = "2.12"):
        if ml_type and ml_type.lower() not in ["cpu", "gpu"]:
            raise ValueError('"ml_type" can only be "cpu" or "gpu"!')
        return "".join(
            [
                f"{major}.",
                f"{feature}.x",
                "" if not ml_type else f"-{ml_type}-ml",
                f"-scala{scala_version}",
            ]
        )

    @staticmethod
    def v2(
        major: str,
        feature: str,
        maintenance: str,
        runtime_version: str,
        scala_version: str = "2.11",
    ):
        raise ValueError("This version of runtime is no longer supported!")

    @staticmethod
    def light(major: str, feature: str, scala_version: str = "2.11"):
        return f"apache-spark.{major}.{feature}.x-scala{scala_version}"



import click
import yaml
from dbks.cluster.controller import ClusterController


ctl = ClusterController()


@click.group()
def cli():
    """Commands for Databricks cluster operations."""
    pass


@cli.command()
@click.argument("yml_path", type=click.Path(exists=True))
@click.option(
    "--pin", type=click.BOOL, default="true", help="Pin cluster", show_default=True
)
def create_or_edit_from_yml(yml_path, pin):
    """\b
    Parse a YAML file and use its content to create or edit cluster(s).
    Please refer to documentation for YAML file example.
    """
    with open(yml_path) as file:
        clusters_to_deploy = yaml.load(file, Loader=yaml.FullLoader)
    for cluster_name, cluster_def in clusters_to_deploy.items():
        ctl.create_or_edit_cluster(cluster_def=cluster_def, pin=pin)


@cli.command()
@click.argument("cluster_name", type=str, required=True)
def delete_by_name(cluster_name):
    """Delete a cluster by name."""
    ctl.delete_by_name(cluster_name)



import base64
from pathlib import Path
from dbks.workspace.api import WorkspaceAPI
from dbks.response_handler import ResponseHandler


class WorkspaceController:
    def __init__(self, api=WorkspaceAPI()):
        if not isinstance(api, WorkspaceAPI):
            raise ValueError("Parameter must be an instance of WorkspaceAPI!")
        self.api = api

    @ResponseHandler
    def delete(self, path, recursive=True):
        return self.api.delete(json={"path": path, "recursive": recursive})

    @ResponseHandler
    def export(self, path, format="SOURCE"):
        return self.api.export(json={"path": path, "format": format})

    @ResponseHandler
    def get_status(self, path):
        return self.api.get_status(json={"path": path})

    @ResponseHandler
    def _import(self, local_path, path, language, overwrite=False, format="SOURCE"):
        # This has a limit of 10 MB
        content = ""
        with open(local_path, "r") as file:
            content = file.read()
        return self.api._import(
            json={
                "content": base64.b64encode(bytes(content, "utf-8")).decode(),
                "path": path,
                "language": language.upper(),
                "overwrite": overwrite,
                "format": format,
            }
        )

    @ResponseHandler
    def list(self, path):
        return self.api.list(json={"path": path})

    @ResponseHandler
    def mkdirs(self, path):
        return self.api.mkdirs(json={"path": path})

    # create folder is not exists
    def imports(self, local_path, path, language, overwrite=False, format="SOURCE"):
        target_path = Path(path)
        self.mkdirs(str(target_path.parent))
        self._import(
            local_path=local_path,
            path=path,
            language=language,
            overwrite=overwrite,
            format=format,
        )

    def get_id_by_path(self, path):
        return self.get_status(path).json()["object_id"]



from dbks.client import Client


class PermissionAPI:
    def __init__(self, client=Client()):
        self.client = client

    # cluster permissions
    def get_cluster_permission_levels(self, params=None, json=None):
        return self.client.call(
            "GET", f"/permissions/clusters/{params['cluster_id']}/permissionLevels"
        )

    def get_cluster_permissions(self, params=None, json=None):
        return self.client.call("GET", f"/permissions/clusters/{params['cluster_id']}")

    def update_cluster_permissions(self, params=None, json=None):
        return self.client.call(
            "PATCH", f"/permissions/clusters/{params['cluster_id']}", json=json
        )

    def replace_cluster_permissions(self, params=None, json=None):
        return self.client.call(
            "PUT", f"/permissions/clusters/{params['cluster_id']}", json=json
        )

    # directory permissions
    def get_directory_permission_levels(self, params=None, json=None):
        return self.client.call(
            "GET", f"/permissions/directories/{params['directory_id']}/permissionLevels"
        )

    def get_directory_permissions(self, params=None, json=None):
        return self.client.call(
            "GET", f"/permissions/directories/{params['directory_id']}"
        )

    def update_directory_permissions(self, params=None, json=None):
        return self.client.call(
            "PATCH", f"/permissions/directories/{params['directory_id']}", json=json
        )

    def replace_directory_permissions(self, params=None, json=None):
        return self.client.call(
            "PUT", f"/permissions/directories/{params['directory_id']}", json=json
        )



import copy
import os
from requests import Session


class Client:
    def __init__(self, version: str = "2.0"):
        self.version = version
        self.host = os.getenv(f"DBC_HOST", None)
        self.token = os.getenv(f"DBC_TOKEN", None)
        self.headers = {"Authorization": f"Bearer {self.token}"}
        self.session = Session()

    @property
    def url(self):
        return f"https://{self.host}/api/{self.version}"

    def call(self, method, endpoint, params=None, json=None, headers=None, **kwargs):
        if headers is None:
            headers = self.headers
        else:
            tmp_headers = copy.deepcopy(self.headers)
            tmp_headers.update(headers)
            headers = tmp_headers
        return self.session.request(
            method,
            self.url + endpoint,
            params=params,
            json=json,
            headers=headers,
            **kwargs,
        )



from dbks.client import Client


class LibraryAPI:
    def __init__(self, client=Client()):
        self.client = client

    def all_cluster_statuses(self, params=None, json=None):
        return self.client.call("GET", "/libraries/all-cluster-statuses")

    def cluster_status(self, params=None, json=None):
        return self.client.call("GET", "/libraries/cluster-status", params=params)

    def install(self, params=None, json=None):
        return self.client.call("POST", "/libraries/install", json=json)

    def uninstall(self, params=None, json=None):
        return self.client.call("POST", "/libraries/uninstall", json=json)



from dbks.cluster.api import ClusterAPI
from dbks.response_handler import ResponseHandler
from dbks.util import same_as_target


class ClusterController:
    def __init__(self, api=ClusterAPI()):
        if not isinstance(api, ClusterAPI):
            raise ValueError("Parameter must be an instance of ClusterAPI!")
        self.api = api

    @ResponseHandler
    def list(self):
        return self.api.list()

    @ResponseHandler
    def get(self, cluster_id):
        return self.api.get(params={"cluster_id": cluster_id})

    @ResponseHandler
    def create(self, cluster_def):
        return self.api.create(json=cluster_def)

    @ResponseHandler
    def edit(self, cluster_id, cluster_def):
        cluster_def["cluster_id"] = cluster_id
        return self.api.edit(json=cluster_def)

    @ResponseHandler
    def start(self, cluster_id):
        return self.api.start(json={"cluster_id": cluster_id})

    @ResponseHandler
    def pin(self, cluster_id):
        return self.api.pin(json={"cluster_id": cluster_id})

    @ResponseHandler
    def unpin(self, cluster_id):
        return self.api.unpin(json={"cluster_id": cluster_id})

    @ResponseHandler
    def delete(self, cluster_id):
        return self.api.delete(json={"cluster_id": cluster_id})

    def get_id_by_name(self, cluster_name=""):
        cluster_ids = []
        if not cluster_name:
            raise ValueError("Please provide cluster name string!")
        res = self.list()
        clusters = res.json()["clusters"]
        for cluster in clusters:
            if cluster["cluster_name"] == cluster_name:
                cluster_ids.append(cluster["cluster_id"])
        return cluster_ids

    def create_or_edit_cluster(self, cluster_def, pin=True):
        cluster_name = cluster_def["cluster_name"]
        cluster_id = None
        cluster_ids = self.get_id_by_name(cluster_name)
        resp = None
        if not cluster_ids:
            print(f'cluster "{cluster_name}" does not exist, creating...')
            resp = self.create(cluster_def)
            cluster_id = resp.json()["cluster_id"]
            if pin:
                self.pin(cluster_id)
        elif len(cluster_ids) == 1:
            print(f'cluster "{cluster_name}" already exists, compare spec...')
            cluster_id = cluster_ids[0]
            current_def = self.get(cluster_id).json()
            if not same_as_target(cluster_def, current_def):
                (f'cluster "{cluster_name}" spec changed, editing...')
                resp = self.edit(cluster_id, cluster_def)
            else:
                print("specs are the same, skipping ...")
            if pin:
                self.pin(cluster_id)
            else:
                self.unpin(cluster_id)
        else:
            raise Exception(
                f'Found multiple clusters {cluster_ids} with name [{cluster_def["cluster_name"]}]!'
            )
        return resp

    def delete_by_name(self, cluster_name=""):
        cluster_ids = self.get_id_by_name(cluster_name)
        if not cluster_ids:
            print(f'cluster "{cluster_name}" does not exist, aborting...')
            return None
        elif len(cluster_ids) == 1:
            print(f'cluster "{cluster_name}" found, deleting...')
            cluster_id = cluster_ids[0]
            self.unpin(cluster_id)
            return self.delete(cluster_id)
        else:
            raise Exception(
                f'Found multiple clusters {cluster_ids} with name "{cluster_name}"!'
            )



from dbks.permission.api import PermissionAPI
from dbks.response_handler import ResponseHandler


class PermissionController:
    def __init__(self, api=PermissionAPI()):
        if not isinstance(api, PermissionAPI):
            raise ValueError("Parameter must be an instance of PermissionAPI!")
        self.api = api

    @ResponseHandler
    def get_cluster_permission_levels(self, cluster_id):
        return self.api.get_cluster_permission_levels(params={"cluster_id": cluster_id})

    @ResponseHandler
    def get_cluster_permissions(self, cluster_id):
        return self.api.get_cluster_permissions(params={"cluster_id": cluster_id})

    @ResponseHandler
    def update_cluster_permissions(self, cluster_id, access_control_list):
        """
        This request only grants (adds) permissions.
        To revoke, use "replace_cluster_permissions".

        "access_control_list" example input:
            [
                {
                    "user_name": "jsmith@example.com",
                    "permission_level": "CAN_RESTART"
                },
                ...
            ]
        """
        return self.api.update_cluster_permissions(
            params={"cluster_id": cluster_id},
            json={"access_control_list": access_control_list},
        )

    @ResponseHandler
    def replace_cluster_permissions(self, cluster_id, access_control_list):
        """
        WARNING: This request overwrites all existing direct (non-inherited)
        permissions on the cluster and replaces it with the new permissions
        specified in the request body.

        "access_control_list" example input:
            [
                {
                    "user_name": "jsmith@example.com",
                    "permission_level": "CAN_RESTART"
                },
                ...
            ]
        """
        return self.api.replace_cluster_permissions(
            params={"cluster_id": cluster_id},
            json={"access_control_list": access_control_list},
        )

    @ResponseHandler
    def get_directory_permission_levels(self, directory_id):
        return self.api.get_directory_permission_levels(
            params={"directory_id": directory_id}
        )

    @ResponseHandler
    def get_directory_permissions(self, directory_id):
        return self.api.get_directory_permissions(params={"directory_id": directory_id})

    @ResponseHandler
    def update_directory_permissions(self, directory_id, access_control_list):
        """
        This request only grants (adds) permissions.
        To revoke, use "replace_directory_permissions".

        "access_control_list" example input:
            [
                {
                    "user_name": "jsmith@example.com",
                    "permission_level": "CAN_READ"
                },
                ...
            ]
        """
        return self.api.update_directory_permissions(
            params={"directory_id": directory_id},
            json={"access_control_list": access_control_list},
        )

    @ResponseHandler
    def replace_directory_permissions(self, directory_id, access_control_list):
        """
        WARNING: This request overwrites all existing direct (non-inherited)
        permissions on the directory and replaces it with the new permissions
        specified in the request body.

        "access_control_list" example input:
            [
                {
                    "user_name": "jsmith@example.com",
                    "permission_level": "CAN_RUN"
                },
                ...
            ]
        """
        return self.api.replace_directory_permissions(
            params={"directory_id": directory_id},
            json={"access_control_list": access_control_list},
        )



import requests
from setuptools import setup, find_packages


def read_requirements():
    with open("requirements.txt", "r") as f:
        content = f.read()
        requirements = content.split("\n")
    return requirements


setup(
    name="dbks",
    version="0.0.1",
    author="Vincent Lam",
    author_email="vincent@lam.is",
    description="Databricks API client",
    long_description="",
    long_description_content_type="text/markdown",
    url="https://github.com/vincentlam/dbks",
    packages=find_packages(),
    install_requires=read_requirements(),
    entry_points="""
        [console_scripts]
        dbks=dbks.cli:cli
    """,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import click
from dbks.workspace.controller import WorkspaceController


ctl = WorkspaceController()


@click.group()
def cli():
    """Commands for Databricks workspace operations."""
    pass


@cli.command()
@click.argument("path")
def mkdirs(path):
    """Create workspace folder."""
    ctl.mkdirs(path)



import click
import yaml
from dbks.cluster.controller import ClusterController
from dbks.permission.controller import PermissionController
from dbks.workspace.controller import WorkspaceController


cluster_ctl = ClusterController()
permission_ctl = PermissionController()
Workspace_ctl = WorkspaceController()


@click.group()
def cli():
    """Commands for Databricks permission operations."""
    pass


@cli.command()
@click.argument("cluster_id")
def get_cluster_permissions(cluster_id):
    permission_ctl.get_cluster_permissions(cluster_id)


@cli.command()
@click.argument("yml_path", type=click.Path(exists=True))
def replace_cluster_permissions_from_yml(yml_path):
    """\b
    Parse a YAML file and use its content to replace cluster permissions.
    Please refer to documentation for YAML file example.
    """
    with open(yml_path) as file:
        target_permissions = yaml.load(file, Loader=yaml.FullLoader)
    for cluster_name, permissions in target_permissions.items():
        cluster_id = cluster_ctl.get_id_by_name(cluster_name)[0]
        permission_ctl.replace_cluster_permissions(cluster_id, permissions)


@cli.command()
@click.argument("yml_path", type=click.Path(exists=True))
def replace_path_permissions_from_yml(yml_path):
    """\b
    Parse a YAML file and use its content to replace folder/notebook permissions.
    Please refer to documentation for YAML file example.
    """
    with open(yml_path) as file:
        target_permissions = yaml.load(file, Loader=yaml.FullLoader)
    for path, permissions in target_permissions.items():
        object_id = Workspace_ctl.get_id_by_path(path)
        permission_ctl.replace_directory_permissions(object_id, permissions)



from typing import Dict


def same_value(src, tgt):
    if type(src) is bool or type(tgt) is bool:
        return str(src).lower() == str(tgt).lower()
    if type(src) is dict and type(tgt) is dict:
        for key in list(src.keys()):
            if key in tgt:
                return same_value(src[key], tgt[key])
            else:
                print(f'Key "{key}" not found in tgt! Flag specs as different.')
                return False
    return src == tgt


def same_as_target(src_dict: Dict, tgt_dict: Dict):
    diff = 0
    if not src_dict or not tgt_dict:
        diff += 1
    for key in list(src_dict.keys()):
        if key in tgt_dict:
            if not same_value(src_dict[key], tgt_dict[key]):
                print(f'Values of key "{key} are different ...')
                diff += 1
        else:
            print(f'Key "{key}" not found in existing cluster specs ...')
            diff += 1
    return True if diff == 0 else False
